classdef conica
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        

Esta 'clase' de Matlab permite definir una cónica bien con los
coeficientes de la ecuación coefs, bien con su matriz en coordenadas ampliada M.

Si se introduce M checkea si M es simétrica (ignora la parte
por debajo de la diagonal)

Calcula automáticamente los autovalores, sus autovectores, la ecuación reducida,
su matriz, la matriz de cambio de referencia Qa,
los parámetros de la cónica (si existen), el tipo de cónica, si es degenerada
o no, focos, centro, vertices.

INSTRUCCIONES

Para crear un nuevo objeto "conica", basta con añadir al PATH de matlab (en el
menu FILE) la ruta a este archivo, y crearlo escribiendo en la consola de
matlab: 

    a = conica

donde "a" es la variable que va a almacenar el objeto.

Para cambiar los coeficientes de la cónica, damos los nuevos coefs en forma 
de vector:

    a.coefs = [1, 2, 3, 4, 5, 6]; % ordenado x^2, y^2, xy, x, y, 1

Si se pregunta cuanto vale "a", nos dará toda la información de la cónica:

    a    % Sin poner ";" al final, para que muestre por pantalla

Si queremos preguntar por un dato en concreto, por ejemplo, la matriz reducida,

    a.Mr % Sin ";" al final

Para dibujar la conica con el resto de elementos, basta con invocar
"dibujar_todo" o sus variantes

    a.dibujar_todo;
    a.dibujar_todo_centrado;
    a.dibujar_todo_con_paraboloide;

Dudas, errores, sugerencias: mario.merino@upm.es

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
MMM20121206
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
    properties
        coefs = [1,1,0,0,0,-1]; % vector de coefs Ax^2 + By^2 + Cxy + Dx +Ey + F = 0
    end
    properties (Dependent = true)
        texto_ecuacion; % mostrar la ecuacion inicial
        texto_ecuacion_r; % mostrar la ecuacion reducida
        x_;
        y_;
        lambda_x_; % autovalores segun esas dos direcciones
        lambda_y_;
        autovector_x_; % autovectores segun esas dos direcciones
        autovector_y_;
        tipo; % el tipo de conica que es: elipse, hiperbola, parabola, punto, conjunto vacio, recta doble, dos rectas paralelas, una recta, dos rectas que se cortan
        es_degenerada; % true/false si es degenerada
        M; % la matriz inicial, en coordenadas ampliadas
        Mr; % la matriz reducida, coordenadas ampliadas
        Qa; % Cambio de referencia desde lo inicial a lo reducido (c. ampliadas)
        a; % Los parámetros de la conica, cuando existan
        b;
        c;
        p; % distancia foco-directriz
        e;
        centro;
        focos;
        vertices;
        ecuacion; % function handles (se usan internamente)
        ecuacion_r;
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    methods
        function v = get.ecuacion(o)
            c = o.coefs;
            v =@(x,y) c(1)*x.^2+c(2)*y.^2+c(3)*x.*y+c(4)*x+c(5)*y+c(6);
        end
        function v = get.ecuacion_r(o)
            c = [o.Mr(2,2),o.Mr(3,3),2*o.Mr(2,3),2*o.Mr(1,2),2*o.Mr(1,3),o.Mr(1,1)];
            v =@(x,y) c(1)*x.^2+c(2)*y.^2+c(3)*x.*y+c(4)*x+c(5)*y+c(6);
        end
        function v = get.texto_ecuacion(o)
            vars = {'x^2','y^2','x*y','x','y'};
            v = [o.expresion(o.coefs,vars),' = 0'];
        end
        function v = get.texto_ecuacion_r(o)
            vars = {'x_^2','y_^2','x_*y_','x_','y_'};
            v = [o.expresion([o.Mr(2,2),o.Mr(3,3),2*o.Mr(2,3),2*o.Mr(1,2),2*o.Mr(1,3),o.Mr(1,1)],vars),' = 0'];
        end
        function v = get.x_(o)
            vars = {'x','y'};
            Qa_ = inv(o.Qa);
            v = o.expresion([Qa_(2,2),Qa_(2,3),Qa_(2,1)],vars);
        end
        function v = get.y_(o)
            vars = {'x','y'};
            Qa_ = inv(o.Qa);
            v = o.expresion([Qa_(3,2),Qa_(3,3),Qa_(3,1)],vars);
        end
        function v = get.lambda_x_(o)
            v = o.Mr(2,2);
        end
        function v = get.lambda_y_(o)
            v = o.Mr(3,3);
        end
        function v = get.autovector_x_(o)
            v = o.Qa(2:3,2);
        end
        function v = get.autovector_y_(o)
            v = o.Qa(2:3,3);
        end
        function v = get.M(o)
            v(3,3) = o.coefs(2); %B
            v(2,2) = o.coefs(1); %A
            v(1,1) = o.coefs(6); %F
            v(2,3) = o.coefs(3)/2; %C/2
            v(3,2) = o.coefs(3)/2; %C/2
            v(1,2) = o.coefs(4)/2; %D/2
            v(2,1) = o.coefs(4)/2; %D/2
            v(1,3) = o.coefs(5)/2; %E/2
            v(3,1) = o.coefs(5)/2; %E/2
        end
        function o = set.M(o,v)
            o.coefs = [v(2,2),v(3,3),2*v(2,3),2*v(1,2),2*v(1,3),v(1,1)];
        end
        function v = get.Mr(o)
            v = conica.ecuacion_reducida(o.M);
        end
        function v = get.Qa(o)
            [~, v] = conica.ecuacion_reducida(o.M);
        end
        function v = get.centro(o)
            v = NaN;
            if any(strcmp(o.tipo,{'elipse','circunferencia','hiperbola'}))
                v = o.Qa(2:3,1);
            end
        end
        function v = get.tipo(o) % esta es la función que clasifica la conica segun su tipo
            Mr = o.Mr;
            detA = o.lambda_x_*o.lambda_y_;
            if detA>0 % forma cuadrática definida, sig(A) = (2,0) o sig(A) = (0,2)
                if Mr(1,1)==0
                    v = 'punto';
                elseif Mr(1,1)*o.lambda_x_<0
                    v = 'elipse';
                    if o.lambda_x_ == o.lambda_y_
                        v = 'circunferencia'; % caso concreto de elipse
                    end
                else
                    v = 'conjunto vacio (elipse imaginaria)';
                end
            elseif detA<0 % forma cuadrática sig(A) = (1,1)
                if Mr(1,1)==0
                    v = 'dos rectas que se cortan';
                else
                    v = 'hiperbola';
                end
            else % al menos un autovalor de A es 0
                if o.lambda_x_ ~= 0 || o.lambda_y_ ~= 0 % el otro autovalor es no nulo
                    if Mr(1,2) ~= 0 || Mr(1,3) ~= 0 % hay parte lineal en Mr
                        v = 'parabola';
                    else % no hay parte lineal
                        if Mr(1,1) == 0 % recta doble
                            v = 'recta doble';
                        elseif Mr(1,1)*(o.lambda_x_ + o.lambda_y_) < 0
                            v = 'dos rectas paralelas';
                        else % no hay solucion
                            v = 'conjunto vacio (dos rectas paralelas imaginarias)';
                        end
                    end
                else % los dos autovalores nulos: no es ecuación cuadrática
                    if Mr(1,2) ~= 0 || Mr(1,3) ~= 0 % hay parte lineal en Mr
                        v = 'una recta simple';
                    else
                        v = 'no hay solucion';
                    end
                end
            end
        end
        function v = get.es_degenerada(o) % cualquier cosa que no sea 'elipse','circunferencia','hiperbola','parabola', o 'conjunto vacio (elipse imaginaria)'
            v = (det(o.Mr)==0);
        end
        % Parametros de la conica: a b c e. Los calculo a partir de los
        % autovalores
        function v = get.a(o)
            v = NaN;
            if any(strcmp(o.tipo,{'elipse','circunferencia','hiperbola'}))
                v = sqrt( abs(max([-o.Mr(1,1)/o.Mr(2,2),-o.Mr(1,1)/o.Mr(3,3)]) ));
            end
            if strcmp(o.tipo,'parabola')
                v = Inf; % por dar un valor a la parabola (ignorad esto)
            end
        end
        function v = get.b(o)
            v = NaN;
            if any(strcmp(o.tipo,{'elipse','circunferencia','hiperbola'}))
                v = sqrt( abs(min([-o.Mr(1,1)/o.Mr(2,2),-o.Mr(1,1)/o.Mr(3,3)]) ));
            end
            if strcmp(o.tipo,'parabola')
                v = sqrt(o.p); % lo pongo por dar algun valor para la parábola (ignorad esto)
            end
        end
        function v = get.c(o)
            v = NaN;
            if any(strcmp(o.tipo,{'elipse','circunferencia','hiperbola'}))
                v = sqrt(abs( o.Mr(1,1)/o.Mr(2,2) - o.Mr(1,1)/o.Mr(3,3) ));
            end
            if strcmp(o.tipo,'parabola')
                v = Inf; % (ignorad esto)
            end
        end
        function v = get.e(o)
            v = NaN;
            if any(strcmp(o.tipo,{'elipse','circunferencia','hiperbola'}))
                v = o.c/o.a;
            end
            if strcmp(o.tipo,'parabola')
                v = 1;
            end
        end        
        function v = get.p(o)        % Parametro p, distancia foco-directriz
            v = NaN;
            if any(strcmp(o.tipo,{'elipse','circunferencia','hiperbola'}))
                v = abs(o.a-o.c)*(1+1/o.e);
            end
            if strcmp(o.tipo,'parabola')
                v = abs((o.Mr(1,2)+o.Mr(1,3))/(o.Mr(2,2)+o.Mr(3,3)));
            end
        end
        function v = get.focos(o)
            % posicion de los focos en ejes iniciales
            switch o.tipo
                case {'elipse','circunferencia'} % los focos estan en la direccion de lambda(2) (el autovalor de menor valor abs)
                    if abs(o.lambda_x_) < abs(o.lambda_y_)
                        temp1 = o.Qa*[1;o.c;0];
                        temp2 = o.Qa*[1;-o.c;0];
                        v = {temp1(2:3),temp2(2:3)};
                    else
                        temp1 = o.Qa*[1;0;o.c];
                        temp2 = o.Qa*[1;0;-o.c];
                        v = {temp1(2:3),temp2(2:3)};
                    end
                case 'hiperbola' % no es cuestión de val abs, sino de signo
                    if o.Mr(1,1)*o.lambda_x_ < 0
                        temp1 = o.Qa*[1;o.c;0];
                        temp2 = o.Qa*[1;-o.c;0];
                        v = {temp1(2:3),temp2(2:3)};
                    else
                        temp1 = o.Qa*[1;0;o.c];
                        temp2 = o.Qa*[1;0;-o.c];
                        v = {temp1(2:3),temp2(2:3)};
                    end
                case 'parabola' % el autovector asociado a 0.
                    if o.lambda_x_ == 0
                        signo = -sign(o.Mr(1,2)*o.lambda_y_);
                        temp1 = o.Qa*[1;signo*[o.p/2;0]];
                        v = {temp1(2:3)};
                    else
                        signo = -sign(o.Mr(1,3)*o.lambda_x_);
                        temp1 = o.Qa*[1;signo*[0;o.p/2]];
                        v = {temp1(2:3)};
                    end
                otherwise
                    v = {};
            end
        end
        function v = get.vertices(o)
            v = {};
            switch o.tipo
                case 'elipse'
                    v{1}  = o.centro + o.autovector_x_*sqrt(-o.Mr(1,1)/o.lambda_x_);
                    v{2}  = o.centro - o.autovector_x_*sqrt(-o.Mr(1,1)/o.lambda_x_);
                    v{3}  = o.centro + o.autovector_y_*sqrt(-o.Mr(1,1)/o.lambda_y_);
                    v{4}  = o.centro - o.autovector_y_*sqrt(-o.Mr(1,1)/o.lambda_y_);
                case 'hiperbola'
                    if o.Mr(1,1)*o.lambda_x_ < 0 
                        v{1}  = o.centro + o.autovector_x_*sqrt(-o.Mr(1,1)/o.lambda_x_);
                        v{2}  = o.centro - o.autovector_x_*sqrt(-o.Mr(1,1)/o.lambda_x_);
                    else
                        v{1}  = o.centro + o.autovector_y_*sqrt(-o.Mr(1,1)/o.lambda_y_);
                        v{2}  = o.centro - o.autovector_y_*sqrt(-o.Mr(1,1)/o.lambda_y_);
                    end
                case 'parabola'
                    v{1}  = o.Qa(2:3,1);
            end
        end
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    methods
        function [x,y] = puntos_conica(o)
            % obtiene puntos de la conica con la parametrización theta-rho
            switch o.tipo
                case {'elipse','circunferencia','hiperbola','parabola'} % conicas ordinarias
                    % parametrizamos la conica en polares centradas en un foco:
                    % r = e*p/(1+e*cos(theta)), con r=0 el foco. 
                    e = o.e;
                    p = o.p;
                    % puntos en polares focales
                    theta = linspace(0,2*pi,200);
                    if e == 0 % caso de la circunferencia
                        r = sqrt(-o.Mr(1,1)/o.Mr(2,2));
                    else
                        r = e*p./(1+e*cos(theta));
                    end
                    % puntos en cartesianas focales
                    x = r.*cos(theta);
                    y = r.*sin(theta);
                                       
                otherwise
                    error('Conica degenerada: no implementado aun');
            end
            % traslacion al los ejes de la ecuacion reducida             
            switch o.tipo % permuta x por y si hace falta                
                case {'elipse','circunferencia'}
                    x = o.c+x;
                    if o.Mr(2,2) > o.Mr(3,3) % focos sobre el eje y
                        temp = x;
                        x = y;
                        y = temp;
                    end
                case 'hiperbola'
                    x = o.c-x; % la directriz esta hacia el centro desde el foco, en el caso de la hiperbola
                    if  o.Mr(1,1)*o.Mr(3,3) < 0 % focos sobre el eje y
                        temp = x;
                        x = y;
                        y = temp;
                    end
                case 'parabola'
                    x = o.p/2+x;
                    signo = sign(-(o.Mr(1,2)+o.Mr(1,3))/o.Mr(2,2)+o.Mr(3,3));
                    x = signo*x; % orientación de la parábola
                    if o.Mr(3,3) == 0  % focos sobre el eje y
                        temp = x;
                        x = y;
                        y = temp;
                    end
            end         
            % finalmente, llevar los puntos a la referencia inicial
            for i = 1:length(x)
                temp = o.Qa*[1,x(i),y(i)]';
                x(i) = temp(2);
                y(i) = temp(3);
            end
        end
        function [XX,YY,FF] = puntos_paraboloide(o)
            % calculo de puntos del paraboloide, 
            % viendo la ecuacion de la conica como z = f(x,y).            
            extra = 5*sqrt(abs(sum(o.Mr(1,1:3))/abs(min([-1,o.lambda_x_,o.lambda_y_])))); % +-extra para los ejes en cada dirección
            if any(strcmp(o.tipo,{'elipse','circunferencia'}))
                extra = extra/5*3;
            end
            [XX,YY] = meshgrid(linspace(-extra,extra,400),linspace(-extra,extra,400));
            FF = o.ecuacion_r(XX,YY);
            % vuelve a referencia inicial
            Qa = o.Qa;            
            temp = Qa(2,2).*XX+Qa(2,3).*YY;
            YY = Qa(3,2).*XX+Qa(3,3).*YY;
            XX = temp;
            XX = XX+Qa(2,1);
            YY = YY+Qa(3,1);
        end
        function lh = dibujar_conica(o,varargin)
            [XX,YY,FF] = o.puntos_paraboloide;
            L = o.contourl(XX,YY,FF,[0,0]);
            lh = [];
            for i=1:length(L)
                lh(i) = line(L(i).x,L(i).y,'Linewidth',2,'Color','r',varargin{:});
            end
        end
        function ah = dibujar_todo_con_paraboloide(o,varargin)
            ah = o.dibujar_todo;
            view([1,1,1]);
            % Dibujar paraboloide, plano, y conica
            [XX,YY,FF] = o.puntos_paraboloide;
            FF(FF>5) = 5; % no muestra los FF mayores que 5 
            FF(FF<-5) = -5; % no muestra los FF menores que -5
            set(gcf,'renderer','OpenGL');
            sh = surface(XX,YY,FF,'LineStyle','none','clipping','on');
            set(sh,'FaceColor','b','Facealpha','interp','AlphaData',1-abs(FF)/5,'Alphadatamapping','none'); % paraboloide
            surface(XX,YY,XX.*0,'FaceAlpha',0.6,'FaceColor','g','Linestyle','none'); % plano de corte                        
            set(ah,'ZLim',[min(FF(:)),max(FF(:))],'ALim',[-5,5]);
        end
        function lh = dibujar_centro(o,varargin)
            lh = [];
            if any(strcmp(o.tipo,{'elipse','circunferencia','hiperbola'}))
                lh = line(o.centro(1),o.centro(2),'Color','r','Marker','o','Linestyle','none',varargin{:});
            end
        end
        function lh = dibujar_ejes(o,varargin)
            X1 = o.Qa(2,1) + 2* sqrt(max([1,abs(o.Mr(1,1))])/max([1,abs(o.lambda_x_)]))...
                 *(o.autovector_x_(1))*[-1,1];
            Y1 = o.Qa(3,1) + 2* sqrt(max([1,abs(o.Mr(1,1))])/max([1,abs(o.lambda_x_)]))...
                 *(o.autovector_x_(2))*[-1,1];
            X2 = o.Qa(2,1) + 2* sqrt(max([1,abs(o.Mr(1,1))])/max([1,abs(o.lambda_y_)]))...
                 *(o.autovector_y_(1))*[-1,1];
            Y2 = o.Qa(3,1) + 2* sqrt(max([1,abs(o.Mr(1,1))])/max([1,abs(o.lambda_y_)]))...
                 *(o.autovector_y_(2))*[-1,1];
             lh(1) = line(X1,Y1,'LineStyle','-','Color','k',varargin{:});
             lh(2) = line(X2,Y2,'LineStyle','-','Color','k',varargin{:});
        end
        function lh = dibujar_focos(o,varargin)
            lh = [];
            for i=1:length(o.focos)
                lh(i) = line(o.focos{i}(1),o.focos{i}(2),'Marker','o','Linestyle','none',varargin{:});
            end
        end
        function lh = dibujar_directrices(o,varargin)
            lh = [];
            switch o.tipo
                case 'elipse'
                    if abs(o.lambda_x_) < abs(o.lambda_y_)
                        eje_ppal = o.autovector_x_;
                        eje_sec = o.autovector_y_;
                    else
                        eje_ppal = o.autovector_y_;
                        eje_sec = o.autovector_x_;
                    end
                case 'hiperbola'
                    if o.lambda_x_*o.Mr(1,1) < 0
                        eje_ppal = -o.autovector_x_;
                        eje_sec = o.autovector_y_;
                    else
                        eje_ppal = -o.autovector_y_;
                        eje_sec = o.autovector_x_;
                    end
                case 'parabola'
                    if o.lambda_x_ == 0
                        eje_ppal = sign(o.Mr(1,2)*o.lambda_y_)*o.autovector_x_;
                        eje_sec = o.autovector_y_;
                    else
                        eje_ppal = sign(o.Mr(1,3)*o.lambda_x_)*o.autovector_y_;
                        eje_sec = o.autovector_x_;
                    end
                otherwise
                    return;
            end
            for i=1:length(o.focos)
                foco = [o.focos{i}(1);o.focos{i}(2)];
                D = foco + (-1)^(i+1)*o.p*eje_ppal;
                X = D(1) + 2*eje_sec(1)*[-1,1];
                Y = D(2) + 2*eje_sec(2)*[-1,1];
                lh(i) = line(X,Y,'Linestyle','--','Color','k',varargin{:});
            end
        end
        function lh = dibujar_asintotas(o,varargin)
            lh = [];
            if strcmp(o.tipo,'hiperbola')
                v_1 = [sqrt(abs(o.lambda_y_));-sqrt(abs(o.lambda_x_))];
                v_1 = v_1/norm(v_1);
                temp = o.Qa*[1;10*v_1];
                P11 = temp(2:3);
                temp = o.Qa*[1;-10*v_1];
                P12 = temp(2:3);
                v_2 = [sqrt(abs(o.lambda_y_));sqrt(abs(o.lambda_x_))];
                v_2 = v_2/norm(v_2);
                temp = o.Qa*[1;10*v_2];
                P21 = temp(2:3);
                temp = o.Qa*[1;-10*v_2];
                P22 = temp(2:3);
                lh(1) = line([P11(1),P12(1)],[P11(2),P12(2)],'Linestyle','--',varargin{:});
                lh(2) = line([P21(1),P22(1)],[P21(2),P22(2)],'Linestyle','--',varargin{:});
            end
        end
        function lh = dibujar_vertices(o,varargin)
            lh = [];
            for i=1:length(o.vertices)
                lh(i) = line(o.vertices{i}(1),o.vertices{i}(2),'Marker','.','Markersize',14,'Linestyle','none','Color','k',varargin{:});
            end
        end
        function ah = dibujar_todo(o)
            ah = gca;
            % centrar los ejes en la conica:
            D = o.Qa(2:3,1); % el centro de la figura
            extra = 4*sqrt(abs(sum(o.Mr(1,1:3))/abs(min([-1,o.lambda_x_,o.lambda_y_])))); % +-extra para los ejes en cada dirección
            if any(strcmp(o.tipo,{'elipse','circunferencia'}))
                extra = extra/4*3;
            end
            set(ah,'DataAspectRatio',[1,1,1],'XLim',D(1)+[-extra,+extra],'YLim',D(2)+[-extra,+extra]);  
            % Dibujar todo
            dibujar_conica(o);
            dibujar_centro(o);
            dibujar_ejes(o);
            dibujar_focos(o);
            dibujar_asintotas(o);
            dibujar_directrices(o);
            dibujar_vertices(o);
        end
        function ah = dibujar_todo_centrado(o)
            o2 = conica;
            o2.M = o.Mr; % crea un nuevo objeto conica y establece M = o.Mr 
            ah = o2.dibujar_todo;
        end
    end    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    methods (Static = true)
        function [Mr, Qa] = ecuacion_reducida(M)
            % Devuelve la forma reducida de la cónica a partir de una forma cualquiera
            % dada en coordenadas ampliadas, Xt*M*X = 0
            %
            %     [  c   b1   b2 ]        [  1 ]
            % M = [ b1  a11  a12 ]    X = [ x1 ]
            %     [ b2  a12  a22 ]        [ x2 ]
            %
            % Última revisión: MMM20121203
            
            % Submatrices A, B, C
            A = M(2:end,2:end);
            B = M(2:end,1);
            C = M(1,1);
            
            % Direcciones principales
            [Q,A] = eig(A);
            B = Q'*B;
            
            % Cambio de origen
            D = [0; 0]; % initialization
            if A(1,1) ~= 0 % el primer autovalor
                D(1) = -B(1)/A(1,1);
                C = C + D(1)*B(1);
                B(1) = 0;
            end
            if A(2,2) ~= 0 % el segundo autovalor
                D(2) = -B(2)/A(2,2);
                C = C + D(2)*B(2);
                B(2) = 0;
            end
            if A(1,1) == 0 && B(1) ~= 0
                D(1) = -C/(2*B(1));
                C = 0;
            end
            if A(2,2) == 0 && B(2) ~= 0
                D(2) = -C/(2*B(2));
                C = 0;
            end
            
            % Matriz de cambio de referencia
            Qa = [1, 0, 0;
                Q*D,  Q];
            % Matriz reducida
            Mr = [C B';
                B A];
        end
        function eq = expresion(coefs,vars)
            % escribe el texto de una ecuacion dados los coefs y vars en orden.
            % El ultimo coef es el termino indep, y no lleva vars asociada.
            % No se añade = 0 al final
            eq = '';
            for i=1:length(vars)
                if coefs(i) == 1
                    if isempty(eq) % inicialmente no poner '+'
                        eq = [eq, vars{i}];
                    else
                        eq = [eq,' + ', vars{i}];
                    end
                elseif coefs(i) == -1
                    eq = [eq,' - ', vars{i}];
                elseif coefs(i) > 0
                    if isempty(eq) % inicialmente no poner '+'
                        eq = [eq,num2str(coefs(i),3),'*',vars{i}];
                    else
                        eq = [eq,' + ',num2str(coefs(i),3),'*',vars{i}];
                    end
                elseif coefs(i) < 0
                    eq = [eq,' - ',num2str(abs(coefs(i)),3),'*',vars{i}];
                end
            end    
            if coefs(end) > 0
                if isempty(eq) % inicialmente no poner '+'
                    eq = [eq,num2str(coefs(end),3)];
                else
                    eq = [eq,' + ',num2str(coefs(end),3)];
                end
            elseif coefs(end) < 0
                eq = [eq,' - ',num2str(abs(coefs(end)),3)];
            end
        end
        function [lines,M] = contourl(varargin)
            % lines is a structure array with x y z l fields. x y z are the line XData YData ZData. l is the
            % contour level
            
            % Let Matlab provide the M matrix
            
            M = contours(varargin{:});
            
            % Go through M array and separate data chunks
            
            i = 0;  % current line element
            ic = 1; % current index
            
            if isempty(M)
                lines = struct('x',{},'y',{},'z',{});
                return;
            end
            
            while 1==1 % always
                
                if ic > length(M(1,:))
                    break;
                end
                
                i = i + 1;
                
                lines(i).l = M(1,ic); % Contour Level saved here
                lines(i).x = M(1,ic+1:ic+M(2,ic));
                lines(i).y = M(2,ic+1:ic+M(2,ic));
                lines(i).z = lines(i).x.*0 + M(1,ic);
                
                ic = ic + M(2,ic) + 1;
                
            end
        end
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
end

